/* Copyright (c) 2013 Asustor Inc. All rights reserved. */
/**
 * Change info
 * @class AS.ARC.apps.miniDLNA.core.changeInfo
 * @extends Ext.container.Container
 * <p></p>
 */
Ext.define('AS.ARC.apps.miniDLNA.core.changeInfo', {
    extend: 'Ext.container.Container',

    alias: ['widget.asMiniDLNAChangeInfo'],

    layout: 'hbox',

    width: 350,

    style: 'opacity:0;',

    imgUrl: 'resources/images/icons/',

    fadeOutId: '',

    applyChanged: function () {
        var me = this,
            ctIems = me.items;

        ctIems.get('icon').setSrc(me.imgUrl + 'ok.png');
        ctIems.get('message').setText(_S('COMMON', 'APPLY_CHANGE'));
        ctIems.get('message').el.dom.setStyle('font-size: 12px; color: #008000;');
        me.doDisplay();
    },

    applyNotChanged: function () {
        var me = this,
            ctIems = me.items;

        ctIems.get('icon').setSrc(me.imgUrl + 'warning2.png');
        ctIems.get('message').setText(_S('COMMON', 'SETTING_NO_CHANGE'));
        ctIems.get('message').el.dom.setStyle('font-size: 12px; color: #cc6633;');

        me.doDisplay();
    },

    resetDisplay: function () {
        var me = this;
        clearTimeout(me.fadeOutId);
        me.el.stopAnimation();
        me.el.dom.style.opacity = 0;
    },

    doDisplay: function () {
        var me = this;
        clearTimeout(me.fadeOutId);
        me.el.stopAnimation();
        me.el.fadeIn();
        me.fadeOutId = Ext.defer(function () {
            if (me.el) {
                me.el.fadeOut({duration: 3000});
            }
        }, 5000);
    },

    initComponent: function () {
        var me = this;
       
        Ext.apply(me, {
            items: [{
                xtype: 'image',
                itemId: 'icon',
                margin: '1 0 0 0',
                width: 16,
                height: 16,
                src: me.imgUrl + 'warning2.png'
            }, {
                xtype: 'label',
                itemId: 'message',
                text: _S('COMMON', 'SETTING_NO_CHANGE'),
                margin: '1 0 0 3',
                fieldStyle: 'color:#cc6633;'
            }]
        });
        me.callParent(arguments);
    },

    beforeDestroy: function () {
        var me = this;
        me.callParent(arguments);
    }
});

/**
 * @class AS.ARC.apps.miniDLNA.core
 * @extends Ext.util.Observable
 * <p>core</p>
 */
Ext.define('AS.ARC.apps.miniDLNA.core', {
    extend: 'Ext.util.Observable',

    apiUrl: AS.ARC.util.getUserAppsPath() + 'minidlna/' + 'minidlna.cgi',
    imgUrl: AS.ARC.util.getUserAppsPath() + 'minidlna/images',

    constructor: function (config) {
        Ext.apply(this, config);
        this.callParent();
        this.init(config);
    },

    init: function () {
        var fn = this;

        //create an miniDLNA app
        this.win = this.desktop.createWindow({
            app: fn.app,
            id: fn.id,
            itemId: fn.id,
            width: 880,
            height: 550,
            resizable: false,
            border: false,
            items: [
                fn.getMainPanel()
            ],
            listeners: {
                afterrender: function (win) {
                    win.header.items.items[1].hide();
                },
                beforeclose: function (win) {
                    var panel = win.items.get('main');
                    if (fn.doCheckDirty(panel) && !panel.isDiscard) {
                        return false;
                    }
                }
            }
        });
    },

    resetForm: function (form) {
        var me = this;

        form.getFields().each(function (f) {
            if (f.resetOriginalValue) {
                f.resetOriginalValue();
            }
        });
        form.checkDirty();
    },
    
    doCheckDirty: function (panel) {
        var fn = this;

        if (panel.getForm().isDirty()) {
            var okHandler = function () {
                panel.isDiscard = true;
                confirmWin.close();
                fn.win.close();
            }

            var confirmWin = Ext.create('AS.ARC.msgWindow', {
                parentWin: fn.win,
                title: _S('COMMON', 'CONFIRMATION'),
                width: 400,
                height: 170,
                iconType: 'question',
                asItems: [{
                    xtype: 'displayfield',
                    value: _S('COMMON', 'CONFIRM_SETTING_MODIFY')
                }],
                fbar: [{
                    text: _S('COMMON', 'OK'),
                    handler: okHandler
                }, {
                    text: _S('COMMON', 'CANCEL'),
                    handler: function () {
                        confirmWin.close();
                    }
                }]
            }).show();

            return true;
        }
    },

    getMainPanel: function () {
        var fn = this;

        var loadData = function (json) {
            var panelItems = panel.items,
                name = panelItems.get('information').items.get('name'),
                interfaceCombo = panelItems.get('information').items.get('interface'),
                data = [],
                isExist = false,
                saveInterface = json.interface;

            name.setValue(json.name);

            Ext.each(json.if_list, function (item) {
                var display = item.if_name + ' - ' + item.ip;
                data.push([display, item.if_name]);

                if (item.if_name === json.interface) {
                    isExist = true;
                }
            });
            if (!isExist) {
                data.push([saveInterface, saveInterface]);
                // for comb validator
                interfaceCombo.lostInterface = saveInterface;
            }
            interfaceCombo.getStore().loadData(data);
            interfaceCombo.setValue(saveInterface);

            fn.resetForm(panel.getForm());
        };

        var gridStore = Ext.create('Ext.data.Store', {
            pageSize: 5,
            autoLoad: true,
            fields: ['path'],
            proxy: {
                type: 'ajax',
                url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'get'}),
                actionMethods: {
                    create: 'POST',
                    read: 'POST',
                    update: 'POST',
                    destroy: 'POST'
                },
                reader: {
                    type: 'json',
                    root: 'folder',
                    total: 'total'
                },
                listeners: {
                    exception: function (proxy, response, operation, opts) {
                        AS.ARC.util.showMsgWindow({
                            
                        }, Ext.JSON.decode(response.responseText), fn.app.win);
                    }
                }
            },
            listeners: {
                beforeload: function () {
                    var grid = panel.items.get('source').items.get('grid');
                    grid.el.mask(_S('COMMON', 'LOADING'));
                },
                load: function (store, records) {
                    var grid = panel.items.get('source').items.get('grid');
                    grid.el.unmask();
                    loadData(store.proxy.reader.jsonData);
                }
            }
        });

        var rescanHandler = function () {
            var okHandler = function () {
                confirmWin.el.mask(_S('COMMON', 'APPLYING'));

                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'del'}),
                    method: 'post',
                    success: function (json, opts) {
                        confirmWin.el.unmask();
                        confirmWin.close();
                    },
                    failure: function (json, opts) {
                        confirmWin.el.unmask();
                        AS.ARC.util.showMsgWindow({

                        }, json, confirmWin);
                    }
                });
            };

            var confirmWin = Ext.create('AS.ARC.msgWindow', {
                title: _S('COMMON', 'CONFIRMATION'),
                parentWin: fn.win,
                width: 400,
                height: 170,
                iconType: 'question',
                asItems: [{
                    xtype: 'displayfield',
                    value: _S('MINIDLNA', 'CONFIRM_RESCAN')
                }],
                fbar: [{
                    text: _S('COMMON', 'OK'),
                    handler: okHandler
                }, {
                    text: _S('COMMON', 'CANCEL'),
                    handler: function () {
                        confirmWin.close();
                    }
                }]
            }).show();
        };

        var addHandler = function () {
            var browseHandler = function () {
                var fileBrowser = Ext.create('AS.ARC.treeSelector', {
                    parentWin: addWin,
                    objectPath: addWin.items.get('main').items.get('hbox').items.get('path')
                });
                fileBrowser.win.show();
            };

            var applyHandler = function () {
                var path = addWin.items.get('main').items.get('hbox').items.get('path');

                addWin.el.mask(_S('COMMON', 'APPLYING'));
                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'folder'}),
                    params: {
                        add: path.getValue()
                    },
                    method: 'post',
                    success: function (json, opts) {
                        var grid = panel.items.get('source').items.get('grid');

                        grid.getStore().load();
                        addWin.el.unmask();
                        addWin.close()
                    },
                    failure: function (json, opts) {
                        addWin.el.unmask();
                        AS.ARC.util.showMsgWindow({

                        }, json, addWin);
                    }
                });
            };

            var addWin = Ext.create('AS.ARC.subWindow', {
                title: _S('COMMON', 'ADD'),
                width: 420,
                height: 150,
                parentWin: fn.win,
                items: [{
                    xtype: 'asPanelTop20',
                    itemId: 'main',
                    buttonApply: {},
                    width: 420,
                    height: 75,
                    items: [{
                        xtype: 'container',
                        itemId: 'hbox',
                        layout: 'hbox',
                        items: [{
                            xtype: 'textfield',
                            itemId: 'path',
                            fieldLabel: _S('COMMON', 'PATH'),
                            labelWidth: 120,
                            width: 300,
                            msgTarget: AS.ARC.config.msgTarget,
                            allowBlank: false,
                            blankText: _S('MESSAGE', 'HF00011'),
                            readOnly: true
                        }, {
                            xtype: 'button',
                            margin: '0 0 0 10',
                            text: _S('COMMON', 'BROWSE'),
                            handler: browseHandler
                        }]
                    }]
                }],
                fbar: [{
                    xtype: 'button',
                    itemId: 'apply',
                    text: _S('COMMON', 'OK'),
                    disabled: true,
                    handler: applyHandler
                }, {
                    xtype: 'button',
                    itemId: 'cancel',
                    text: _S('COMMON', 'CANCEL'),
                    handler: function () {
                        addWin.close();
                    }
                }],
                listeners: {
                    afterrender: function (win) {
                        var panel = win.items.get('main');
                        panel.buttonApply = win.dockedItems.items[1].items.get('apply');
                    }
                }
            }).show();
        };

        var removeHandler = function () {
            var okHandler = function () {
                var select = panel.items.get('source').items.get('grid').getSelectionModel().getSelection();

                confirmWin.el.mask(_S('COMMON', 'APPLYING'));
                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'folder'}),
                    params: {
                        del: select[0].data.path
                    },
                    method: 'post',
                    success: function (json, opts) {
                        var grid = panel.items.get('source').items.get('grid');

                        confirmWin.el.unmask();
                        confirmWin.close();
                        grid.getStore().load();
                    },
                    failure: function (json, opts) {
                        confirmWin.el.unmask();
                        AS.ARC.util.showMsgWindow({

                        }, json, confirmWin);
                    }
                });
            };

            var confirmWin = Ext.create('AS.ARC.msgWindow', {
                title: _S('COMMON', 'CONFIRMATION'),
                parentWin: fn.win,
                width: 400,
                height: 170,
                iconType: 'question',
                asItems: [{
                    xtype: 'displayfield',
                    value: _S('COMMON', 'REMOVE_SELECTED_ITEMS')
                }],
                fbar: [{
                    text: _S('COMMON', 'OK'),
                    handler: okHandler
                }, {
                    text: _S('COMMON', 'CANCEL'),
                    handler: function () {
                        confirmWin.close();
                    }
                }]
            }).show();
        };

        var applyHandler = function () {
            var infoItems = panel.items.get('information').items,
                name = infoItems.get('name'),
                interfaceCombo = infoItems.get('interface'),
                info = panel.items.get('fbar').items.get('info');

            if (!panel.getForm().isDirty()) {
                info.applyNotChanged();
                return;
            } else {
                info.resetDisplay();
            }

            fn.win.el.mask(_S('COMMON', 'APPLYING'));
            AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'set'}),
                params: {
                    name: name.getValue(),
                    interface: interfaceCombo.getValue()
                },
                method: 'post',
                success: function (json, opts) {
                    var grid = panel.items.get('source').items.get('grid');

                    fn.win.el.unmask();
                    grid.getStore().load();
                    info.applyChanged();
                },
                failure: function (json, opts) {
                    fn.win.el.unmask();
                    AS.ARC.util.showMsgWindow({

                    }, json, fn.win);
                }
            });
        };

        var panel = Ext.create('AS.ARC.formBase', {
            itemId: 'main',
            cls: 'as-page-panel',
            style: 'padding: 0px;',
            border: false,
            height: 512,
            buttonApply: {},
            isDiscard: false,
            items: [{
                xtype: 'fieldset',
                itemId: 'information',
                title: AS.ARC.util.fontToBold(_S('MINIDLNA', 'INFORMATION')),
                margin: '8 15 0 15',
                defaults: {
                    labelWidth: 185,
                    msgTarget: AS.ARC.config.msgTarget
                },
                items: [{
                    xtype: 'textfield',
                    itemId: 'name',
                    width: 400,
                    fieldLabel: AS.ARC.util.fontToBold(_S('MINIDLNA', 'SERVER_NAME')),
                    maxLength: 32,
                    maxLengthText: Ext.String.format(_S('MESSAGE', 'HF00016'), '32'),
                    allowBlank: false,
                    blankText: _S('MESSAGE', 'HF00011')
                }, {
                    xtype: 'combo',
                    itemId: 'interface',
                    width: 400,
                    fieldLabel: AS.ARC.util.fontToBold(_S('NETWORK', 'BONDING_INTERFACE')),
                    store: Ext.create('Ext.data.ArrayStore', {
                        fields: ['display', 'value'],
                        data: []
                    }),
                    displayField: 'display',
                    valueField: 'value',
                    queryMode: 'local',
                    triggerAction: 'all',
                    editable: false,
                    value: 0,
                    lostInterface: '',
                    validator: function (value) {
                        var me = this;
                        if (value === me.lostInterface) {
                            return Ext.String.format(_S('MINIDLNA', 'LAN_LOST'), value);
                        } else {
                            return true;
                        }
                    }
                }, {
                    xtype: 'button',
                    itemId: 'rescan',
                    text: _S('NETWORK', 'RESCAN'),
                    handler: rescanHandler
                }]
            }, {
                xtype: 'fieldset',
                itemId: 'source',
                title: AS.ARC.util.fontToBold(_S('MINIDLNA', 'MEDIA_SOURCE')),
                margin: '15 15 10 15',
                items: [{
                    xtype: 'container',
                    itemId: 'hbox',
                    layout: 'hbox',
                    items: [{
                        xtype: 'button',
                        itemId: 'add',
                        text: _S('COMMON', 'ADD'),
                        margin: '0 0 0 3',
                        handler: addHandler
                    }, {
                        xtype: 'button',
                        itemId: 'remove',
                        text: _S('COMMON', 'REMOVE'),
                        margin: '0 0 0 3',
                        disabled: true,
                        handler: removeHandler
                    }]
                }, {
                    xtype: 'grid',
                    itemId: 'grid',
                    store: gridStore,
                    margin: '5 5 15 5',
                    height: 180,
                    border: false,
                    sortableColumns: false,
                    style: {
                        border: '#BBB 1px solid'
                    },
                    columns: [{
                        header: _S('COMMON', 'PATH'),
                        dataIndex: 'path',
                        menuDisabled: true,
                        flex: 1,
                        renderer: function (value, metadata, record) {
                            metadata.tdAttr = 'data-qtip="' + value + '"';
                            return value;
                        }
                    }],
                    bbar: Ext.create('AS.ARC.pagingToolbar', {
                        store: gridStore
                    }),
                    listeners: {
                        selectionchange: function () {
                            var sourceItems = panel.items.get('source').items,
                                count = sourceItems.get('grid').getSelectionModel().getCount(),
                                remove = sourceItems.get('hbox').items.get('remove');

                            if (count > 0) {
                                remove.enable();
                            } else {
                                remove.disable();
                            }
                        }
                    }
                }]
            }, {
                xtype: 'container',
                itemId: 'fbar',
                layout: {
                    type: 'hbox'
                },
                width: 850,
                margin: '15 15 15 15',
                items: [{
                    xtype: 'asMiniDLNAChangeInfo',
                    itemId: 'info'
                }, {
                    xtype: 'tbfill'
                }, {
                    xtype: 'button',
                    itemId: 'apply',
                    text: _S('COMMON', 'APPLY'),
                    width: 75,
                    style: 'background-image: none;',
                    handler: applyHandler
                }]
            }],
            listeners: {
                afterrender: function (panel) {
                    panel.buttonApply = panel.items.get('fbar').items.get('apply');
                    panel.isDiscard = false;
                }
            }
        });

        return panel;
    }
});

/**
 * @class AS.ARC.apps.miniDLNA.main
 * @extends AS.ARC._appBase
 * <p>Main app</p>
 */
Ext.define('AS.ARC.apps.miniDLNA.main', {
    extend: 'AS.ARC._appBase',

    appTag: 'minidlna',
    title: 'MiniDLNA',
    appMaxNum: 1,
    appOpenNum: 0,
    appIsReady: true,
    appWins: [],

    createWindow: function () {
        var desktop = this.core.getDesktop(),
            app = this;

        if ((this.appOpenNum === this.appMaxNum) || !this.appIsReady) {
            this.appWins[0].show();
            return;
        }

        // Set appIsReady to false
        this.appIsReady = false;

        var miniDLNA = Ext.create('AS.ARC.apps.miniDLNA.core', {
            app: this,
            desktop: desktop,
            id: this.id + '-' + Ext.id()
        });

        miniDLNA.win.on('render', function () {
            app.appOpenNum++;
            app.appIsReady = true;
        });

        miniDLNA.win.on('beforeclose', function () {
            app.appOpenNum--;
            app.appIsReady = true;
            app.appWins.pop();
        });

        miniDLNA.win.show();
        // Get all wins for handling
        this.appWins.push(miniDLNA.win);
        return miniDLNA.win;
    }
});

